@@ -16,11 +16,11 @@
 #
 
 #
-# This edition is a part of netatalk 3.1.8.
+# This edition is a part of netatalk 3.1.11.
 #
 
 use strict;
-use IO::Socket;			# sucks because Timeout doesn't
+use IO::Socket::INET6;			# sucks because Timeout doesn't
 
 my ($arg);
 my ($hostport);
@@ -29,28 +29,34 @@ my ($port);
 
 while ($arg = shift @ARGV)
 {
+	$main::ipv = AF_INET if ($arg eq "-4");
+	$main::ipv = AF_INET6 if ($arg eq "-6");
 	$main::show_icon = 1 if ($arg eq "-i");
 	$main::debug = 1 if ($arg eq "-d");
 	$main::hexdump = 1 if ($arg eq "-x");
 	$main::showver = 1 if ($arg eq "-v");
 	$main::showver = 1 if ($arg eq "-version");
 	$main::showver = 1 if ($arg eq "--version");
-	$hostport = $arg if ($arg !~ /^-/);
+	$port = $arg if (($arg !~ /^-/) && ($hostport));
+	$hostport = $arg if (($arg !~ /^-/) && !($hostport));
 }
 
 if ($main::showver ==1)
 {
         print "$0\n";
         print "Original edition: 7 May 1997 \(v1.0\) James W. Abendschan\n";
-        print "This edition is a part of Netatalk 3.1.8\n";
+        print "This edition is a part of Netatalk 3.1.11\n";
         exit(-1);
 }
 
 if ($hostport eq "")
 {
-	print "usage: $0 [-d] [-i] [-x] hostname[:port]\n";
+	print "usage: $0 [-4|-6] [-d] [-i] [-x] hostname [port]\n";
+	print "       $0 [-4|-6] [-d] [-i] [-x] hostname:port\n";
 	print "       $0 -v|-version|--version\n";
 	print "Queries AFP servers for their capabilities.\n";
+	print "  -4: Use IPv4 only.\n";
+	print "  -6: Use IPv6 only.\n";
 	print "  -d: Enable debug output.\n";
 	print "  -i: Show icon if it exists.\n";
 	print "  -x: Enable hex dump output.\n";
@@ -58,13 +64,52 @@ if ($hostport eq "")
 	exit(-1);
 }
 
-($host, $port) = split(/\:/, $hostport);
-$port = "548" if ($port eq "");
+($host, $port) = parse_hostport($hostport, $port);
 
 my ($packet) = build_packet();
 my ($code) = sendpacket($host, $port, $packet);
 exit $code;
 
+sub parse_hostport
+{
+	my ( $hostport, $port ) = @_;
+	my ($host);
+
+	if (($port) && ($hostport =~ /\[\S+\]/)) # [fd01:1::1] 10548
+	{
+		$host = $hostport;
+	}
+	elsif (($port) && ( $hostport =~ /:/)) # fd01:1::1 10548
+	{
+		$host = "\[" . $hostport . "\]";
+	}
+	elsif ($port) # myserver 10548 / 192.168.0.1 10548
+	{
+	    $host = $hostport;
+	}
+	elsif ($hostport =~ /\[(\S+)\]:(\S+)/) # [fd01:1::1]:10548
+	{
+		$host = "\[" . $1 . "\]";
+		$port = $2;
+	}
+	elsif ($hostport =~ /\[\S+\]/) # [fd01:1::1]
+	{
+		$host = $hostport;
+		$port = "548";
+	}
+	elsif ( (() = $hostport =~ m/:/g) >= 2 ) # fd01:1::1
+	{
+		$host = "\[" . $hostport . "\]";
+		$port = "548";
+	}
+	else # myserver:10548 / myserver / 192.168.0.1:10548 / 192.168.0.1
+	{
+		($host, $port) = split(/\:/, $hostport);
+		$port = "548" if ($port eq "");
+	}
+
+	return ( $host, $port );
+}
 
 sub build_packet 
 {
@@ -92,7 +137,8 @@ sub sendpacket
 
 	print "opening $host:$port\n" if ($main::debug);
 
-	my ($asip_sock) = IO::Socket::INET->new( 
+	my ($asip_sock) = IO::Socket::INET6->new( 
+		Domain => $main::ipv,
 		PeerAddr => $host, 
 		PeerPort => $port, 
 		Proto => 'tcp', 
@@ -131,7 +177,19 @@ sub sendpacket
 	}
 	else
 	{
-		print "AFP reply from $host:$port\n";
+		print "AFP reply from $host:$port";
+		if ($asip_sock->sockdomain == PF_INET)
+		{
+			print " via IPv4\n";
+		}
+		elsif ($asip_sock->sockdomain == PF_INET6)
+		{
+			print " via IPv6\n";
+		}
+		else
+		{
+			print " via unknown IP version\n";
+		}
 		return (parse_packet($buf));
 	}	
 }
@@ -298,6 +356,27 @@ sub parse_FPGetSrvrInfo()
 			my ($utf8name) = extract(1, $utf8_name_offset+1, @packet);
 			print "UTF8 Servername: $utf8name\n";
 		}
+		$offset +=2;
+	}
+
+	if ($flags & (1<<12)) { # Undocumented Bit12
+		# looks like a SPNEGNO negTokenInit blob with the extensions from MS-SPNG
+		# <http://download.microsoft.com/download/9/5/E/95EF66AF-9026-4BB0-A41D-A4F81802D92C/[MS-SPNG].pdf>
+		# <https://tools.ietf.org/html/rfc4178#section-4.2.1>
+		my ($gss_uam_spnego_blob_offset) = unpack("n2", @packet[$offset] . @packet[$offset+1]);
+		print "GSS-UAM SPNEGO blob offset: $gss_uam_spnego_blob_offset\n" if ($main::debug);
+		if ($gss_uam_spnego_blob_offset)
+		{
+			# the leading 4-bytes mean length maybe...
+			my ($gss_uam_spnego_blob_len) = unpack("N2",
+				@packet[$gss_uam_spnego_blob_offset] . @packet[$gss_uam_spnego_blob_offset+1] .
+				@packet[$gss_uam_spnego_blob_offset+2] . @packet[$gss_uam_spnego_blob_offset+3]);
+			my ($gss_uam_spnego_blob) = join("",
+				@packet[$gss_uam_spnego_blob_offset+4..$gss_uam_spnego_blob_offset+4+$gss_uam_spnego_blob_len-1]);
+			print "GSS-UAM SPNEGO blob length: $gss_uam_spnego_blob_len\n" if ($main::debug);
+			print "GSS-UAM SPNEGO blob (Undocumented):\n";
+			hexdump($gss_uam_spnego_blob);
+		}
 	}
 
 	draw_icon($icon_offset, @packet) if ($main::show_icon && $icon_offset);
@@ -424,9 +503,9 @@ sub parse_afp_flags
 	push (@flags, "    SupportsUTF8Servername") if ($flags & (1<<9));
 	push (@flags, "    SupportsUUIDs") if ($flags & (1<<10));
 	push (@flags, "    SupportsExtSleep") if ($flags & (1<<11));
-	push (@flags, "    UnknownBit12") if ($flags & (1<<12));
-	push (@flags, "    UnknownBit13") if ($flags & (1<<13));
-	push (@flags, "    UnknownBit14") if ($flags & (1<<14));
+	push (@flags, "    Undocumented Bit12 (Supports GSS-UAM SPNEGO blob)") if ($flags & (1<<12));
+	push (@flags, "    Undocumented Bit13") if ($flags & (1<<13));
+	push (@flags, "    Undocumented Bit14") if ($flags & (1<<14));
 	push (@flags, "    SupportsSuperClient") if ($flags & (1<<15));
 
 	return @flags;
@@ -505,7 +584,7 @@ sub extract_network_address
 		if ($type == 1)
 		{
 			# quad
-			my ($ip) = sprintf "%d.%d.%d.%d (TCP/IP address)",
+			my ($ip) = sprintf "%d.%d.%d.%d (IPv4 address)",
 				$nap[0], $nap[1], @nap[2], @nap[3];
 	
 			print "Network address: $ip\n";
@@ -515,7 +594,7 @@ sub extract_network_address
 			# quad+port
 			my ($ipport) = sprintf "%d.%d.%d.%d:%d",
 				@nap[0], @nap[1], @nap[2], @nap[3], (@nap[4]*256 + @nap[5]);
-			print "Network address: $ipport (TCP/IP address and port)\n";
+			print "Network address: $ipport (IPv4 address and port)\n";
 		}
 		elsif ($type == 3)
 		{
@@ -534,12 +613,12 @@ sub extract_network_address
 		}
 		elsif ($type == 6 || $type == 7)
 		{
-			print "Network address: ";
+			print "Network address: \[";
 			my ($j) = 0;
 			for ( $j = 0; $j<=13; $j = $j+2) {
 				printf("%.2x%.2x:", @nap[$j], @nap[$j+1]);
 			}
-			printf("%.2x%.2x", @nap[14], @nap[15]);
+			printf("%.2x%.2x\]", @nap[14], @nap[15]);
 			if ($type == 7 ) {
 				printf(":%d", (@nap[16]*256) + @nap[17]);
 				print " (IPv6 address + port)\n";
