@@ -33,6 +33,10 @@
 #include <sys/extattr.h>
 #endif
 
+#ifdef SOLARIS
+#include <sys/attr.h>
+#endif
+
 /* FIXME: are the ACL includes really neccessary here ? */
 #ifdef HAVE_SOLARIS_ACLS
 #include <sys/acl.h>
@@ -54,10 +58,10 @@
 #define MAX_EA_SIZE 3802
 
 /*
- * At time of writing the 10.5.6 client adds 8 bytes to the
- * length of the EA that we send him 
-*/
-#define MAX_REPLY_EXTRA_BYTES 8
+ * req_count has space for AFP response bitmap and length as well, so
+ * 6 bytes
+ */
+#define MAX_REPLY_EXTRA_BYTES 6
 
 /* 
  * Library user must provide a static buffer of size ATTRNAMEBUFSIZ.
@@ -77,6 +81,11 @@ enum {
 #define XATTR_REPLACE 0x2       /* set value, fail if attr does not exist */
 #endif
 
+#ifdef SOLARIS
+#define SMB_ATTR_PREFIX "SUNWsmb:"
+#define SMB_ATTR_PREFIX_LEN (sizeof (SMB_ATTR_PREFIX) - 1)
+#endif
+
 /* Names for our Extended Attributes adouble data */
 #define AD_EA_META "org.netatalk.Metadata"
 #define AD_EA_RESO "org.netatalk.ResourceFork"
